# This work provides a network analysis of exposure to perchlorate 
# at a hypothetical site.  
# 
# The site description:
# A constructed wetland consisting of a 1 hectare (10,000 m2) 
# circular pond with an average depth of 1m and the land in a 
# circle immediately around it.  The total area of the land around 
# the pond, which is a ring shape, is also 1 hectare, implying that 
# the width of the ring is 23.3695 m.  We will assume a turnover rate
# in the pond of 10 days
# 
# Pond radius: 56.41896 m, pond volume: 10,000 m3 (assume 1 m depth)
# Soil ring width: 23.3695, soil volume:10,000 m3 (assume 1 m depth)
# Total radius: 79.78846, total volume: 20,000 m3 (assume 1 m depth)
# 
# System inputs of perchlorate include:
#   - Atmospheric deposition
#   - Seepage through groundwater from nearby disposal site
#   - surface water runoff
# 
# Plant groups include:
#   - aquatic vegetation - volume 900 m3
#   - terrestrial grasses - volume = 500 m3
#   - terrestrial shrubs - volume = 1000 m3
# 
# Animal groups include:
#   - Fish
#   - Amphibians
#   - Birds
#   - Small mammals
#   - Aquatic invertebrates
#   - Humans --> wetland is not a water source for human populations, 
#     but consumption is possible through fishing 
# 
# Network nodes:
#   1) Surface water input node
#   2) Atmospheric deposition input node
#   3) Nearby disposal input node
#   4) Pond water
#   5) Ground water
#   6) Soil
#   7) Aquatic plants
#   8) Terrestrial grasses
#   9) Terrestrial shrubs

###########################################################################
##################### 1) load packages and functions ######################
###########################################################################

#load required packages
library(enaR)
library(limSolve)
library(MASS)
library(network)
library(deSolve)

#set working directory
setwd("C:/Users/dhines/Desktop/AEPint")

#source in new functions
source('./code/functions_ER.R')

#####################################################################
############### clo4 network; Initialization network ################
#####################################################################

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#Note, the parameter values used in this network 
#do not affect analysis results and are only 
#present to facilitate the construction of ODEs 
#for this specific network structure
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#---------------------------
# define boundary inputs (units = g/d)
#---------------------------
#FOR INITIALIZATION ONLY, VALUES DO NOT AFFECT OUTPUT
z1 = 11000 # bottom = 75% ; top = 125%              # surface water input
z2.water = 10e-5 # bottom = 10e-6 ; top = 10e-4     # atmospheric deposition to water
z2.soil = 10e-5 # bottom = 10e-6 ; top = 10e-4      # atmospheric deposition to soil
z3 = 20300#203000 # bottom = 75% ; top = 125%             # disposal input to groundwater

z.original = c(z1,(z2.water+z2.soil),z3,0,0,0,0,0,0,0)

#---------------------------
# define boundary outputs (units = g/d)
#---------------------------
#FOR INITIALIZATION ONLY, VALUES DO NOT AFFECT OUTPUT
y4 = 14045                                             # up to 20% loss from surface water
y5 = 10150 # bottom = 1 ; top = 20300                 # up to 10% loss from groundwater

y.original=c(0,0,0,y4,y5,0,0,0,0)

#---------------------------
# define internal flows (units = g/d)
#---------------------------
#note: row to column orientation, f14 = flow from node 1 to node 4

#FOR INITIALIZATION ONLY, VALUES DO NOT AFFECT OUTPUT
f1.4 = 12000 # bottom = 11000 ; top = 20000             # totally hypothetical
#f1.4 = 11000 # bottom = 75% ; top = 125%               # surface water input
f2.4 = 10e-5 # bottom = 10e-6 ; top = 10e-4             # atmospheric deposition to water
f2.6 = 10e-5 # bottom = 10e-6 ; top = 10e-4             # atmospheric deposition to soil
f3.5 = 20300#203000 # bottom = 75% ; top = 125%               # disposal input to groundwater <-10x less GW input, makes more sense
f4.5 = 58129.9 # bottom = 27578.57 ;  top = 88681.24    # pond water to aquatic plants
f4.7 = 34070.1 # bottom = 26568.76 ; top = 41571.43     # water to aquatic plants
f5.4 = 81200 # bottom = 60900 ; top = 101500            # ground water to pond water
f5.6 = 81200 # bottom = 60900 ; top = 101500            # ground water to soil
f5.8 = 1522.5 # bottom = 1015 ; top = 2030              # groundwater to grasses
f5.9 = 1522.5 # bottom = 1015 ; top = 2030              # groundwater to shrubs
f6.5 = 81200 + 10e-5 # bottom = 60899 ; top = 101501    # soil to ground water

#add in flows from plants back to surface water
f7.4 = 34070.1 # bottom = 26568.76 ; top = 41571.43     # output from aquatic plants
f8.6 = 1522.5 # bottom = 1015 ; top = 2030              # goutput from grasses
f9.6 = 1522.5 # bottom = 1015 ; top = 2030              # output from shrubs

f.original=matrix(c(0,0,0,f1.4,0,0,0,0,0,
                    0,0,0,f2.4,0,f2.6,0,0,0,
                    0,0,0,0,f3.5,0,0,0,0,
                    0,0,0,0,f4.5,0,f4.7,0,0,
                    0,0,0,f5.4,0,f5.6,0,f5.8,f5.9,
                    0,0,0,0,f6.5,0,0,0,0,
                    0,0,0,f7.4,0,0,0,0,0,
                    0,0,0,0,0,f9.6,0,0,0,
                    0,0,0,0,0,f9.6,0,0,0), nrow=9, ncol=9, byrow=TRUE)

#storages by sum of inputs;  this needs to be updated with actual data
#FOR INITIALIZATION ONLY, VALUES DO NOT AFFECT OUTPUT
x.original = c((z1),
               (z2.water+z2.soil),
               (z3),
               (f1.4+f2.4+f5.4),
               (f3.5+f4.5+f6.5),
               (f2.6+f5.6),
               (f4.7),
               (f5.8),
               (f5.9))

#---------------------------
# generate parameterizations
#---------------------------
M.unbalanced=pack(flow=f.original, 
                  input=z.original, 
                  output=y.original, 
                  export=y.original, 
                  respiration = rep(0,9), 
                  storage=x.original, 
                  living=c(rep(FALSE,6),rep(TRUE,3)))

###########################################################################
################ 3) Generate balanced network via ODE #################
###########################################################################

#extract ODE inputs from network model
ODEin=ODEinputs(M.unbalanced) 

#generate function for mass-balance equations
func=ena2ode(M.unbalanced) #stores function as a text string
model=eval(parse(text = func)) #creates a function for use in ode()

#supply a time vector for the ODE solver
times=seq(from=0, to=300, by=0.1)

#solve ODEs to generate steady state
out=ode(y=ODEin$state, times=times, func=model, parms=ODEin$parms)

#visualize results
# my.col=c(NA,'gray','purple','blue','red','black','darkgreen','green3','orange')
# #x11() #windows()
# plot(out[,1],out[,5], type='l', ylim=c(0,200000), 
#      col=my.col[4], xlab='Time', ylab='Storage', 
#      main='ODE-balancing of network model')
# for(j in 6:ncol(out)){
#   lines(out[,1],out[,j],col=my.col[(j-1)], lty=1)
# }
# legend('topleft', legend=c('Surface water','Groundwater','Soil','Grasses','Shrubs','Aquatic plants'), col=my.col[4:9], lty=1, bty='n')

#convert steady state ODE to balanced network
M = odeBack2ena(out=out, parms=ODEin$parms)

#verify that new model is at steady state
SS = ssCheck(M)
if(SS == FALSE){
  print('Network is not at steady state')
}else{
  print('Network is at steady state')
}

###########################################################################
######################### 4) Monte Carlo Analysis #########################
###########################################################################


#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#define distribution for each edge
#note: these values are used to calculate the range of
#rate constants reported in the manuscript
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#f001004
f001004.sample = function(x){
  #uniform distribution
  min = 14
  max = 28
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f002004
f002004.sample = function(x){
  #uniform distribution
  min = 2.340997e-08
  max = 1.511473e-05
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f005004
f005004.sample = function(x){
  #uniform distribution
  min = 1.041226
  max = 3.878749
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f003005
f003005.sample = function(x){
  #uniform distribution
  min = 46
  max = 76
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f004005
f004005.sample = function(x){
  #uniform distribution
  min = 0.1
  max = 1.5
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f006005 # bottom = 60899 ; top = 101501
f006005.sample = function(x){
  #uniform distribution
  min =  2
  max = 10
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f002006
f002006.sample = function(x){
  #uniform distribution
  min = 1.744670e-07
  max = 1.346477e-05
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f005006
f005006.sample = function(x){
  #uniform distribution
  min = 0.0003847341
  max = 0.0010785988
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f004007
f004007.sample = function(x){
  #uniform distribution
  min = 0.18
  max = 1.31
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f005008
# bottom = 1015 ; top = 2030
f005008.sample = function(x){
  min = 5
  max = 25
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f005009
# bottom = 1015 ; top = 2030
f005009.sample = function(x){
  #uniform distribution
  min = 4
  max = 15
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#!! connections back to surface water
#f007004
f007004.sample = function(x){
  #uniform distribution
  min = 0.18
  max = 1.31
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f008006
f008006.sample = function(x){
  #uniform distribution
  min = 15
  max = 35
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#f009006
f009006.sample = function(x){
  #uniform distribution
  min = 8
  max = 29
  parm =  runif(1, min=min, max=max)
  return(parm)
}

#------ SCENARIOS ------

#multiply all valuew by 1 for mild scenario
#multiply all values by 10 for moderate scenario
#multiply z001 and z002 by 10, but z003 by 100 for high scenario

#z001
z001.sample = function(x){
  #uniform distribution
  min = 14 #multiply by 10 for moderate scencario
  max = 28
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#z002
z002.sample = function(x){
  #uniform distribution
  min = 20e-8 
  max = 20e-6 
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#z003
z003.sample = function(x){
  #uniform distribution
  min = 45
  max = 78
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#-----------------------

#e004
e004.sample = function(x){
  #uniform distribution
  min = 15 + 0.1998259 # plus aquatic plant outputs
  max = 30 + 1.3085643 # plus aquatic plant outputs
  parm =  runif(1, min=min, max=max)
  return(parm)
  
}
#e005
e005.sample = function(x){
  #uniform distribution
  min = 15 + 15 + 8 # plus terrestrial plant outputs
  max = 42 + 35 + 29 # plus terrestrial plant outputs
  parm =  runif(1, min=min, max=max)
  return(parm)
  
}
#storage001
X001.sample = function(x){
  min = 10
  max = 33
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage002
X002.sample = function(x){
  min = 1e-07
  max = 2e-05
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage003
X003.sample = function(x){
  min = 40
  max = 115
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage004
X004.sample = function(x){
  min = 1.5
  max = 10
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage005
X005.sample = function(x){
  min = 11
  max = 30
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage006
X006.sample = function(x){
  min = 1 #1.7e-04
  max = 4 #6.6e-04
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage007
X007.sample = function(x){
  min = 1.5
  max = 10
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage008
X008.sample = function(x){
  min = 35
  max = 275
  parm =  runif(1, min=min, max=max)
  return(parm)
}
#storage009
X009.sample = function(x){
  min = 33
  max = 213
  parm =  runif(1, min=min, max=max)
  return(parm)
}

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#set up initial conditions for analysis
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#set up a modified parameter vector of the now-balanced model
ODEin = ODEinputs(M)
parms.mod = ODEin$parms

#note, initialization values do not affect output
state.UA = c("X001" = 19.5105, "X002" = 2.581755e-06, "X003" = 68.5346, "X004" = 4.419719,
             "X005" = 19.70571, "X006" = 0.0003886605, "X007" = 0.3360858, "X008" = 98.41085, "X009" = 90.61475)

#generate function for mass-balance equations
func=ena2ode(M) #stores function as a text string
model=eval(parse(text = func)) #creates a function for use in ode()

#supply a time vector for the ODE solver
times=seq(from=0, to=200, by=0.1)

#number of uncertainty iterations
iter=10000

#initialize output
M.UA = list()
parms.store=list()
out.store=list()
final.out.store=list()
out.mod=0
M.mod=0

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#monte-carlo analysis
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

for(i in 1:iter){
  #generate modified parms set
  parms.mod = ODEin$parms
  parms.mod['f001004'] = f001004.sample(1)
  parms.mod['f002004'] = f002004.sample(1)
  parms.mod['f005004'] = f005004.sample(1)
  parms.mod['f006005'] = f006005.sample(1)
  parms.mod['f002006'] = f002006.sample(1)
  parms.mod['f005006'] = f005006.sample(1)
  parms.mod['f003005'] = f003005.sample(1)
  parms.mod['f004005'] = f004005.sample(1)
  parms.mod['f004007'] = f004007.sample(1)
  parms.mod['f005008'] = f005008.sample(1)
  parms.mod['f005009'] = f005009.sample(1)
  parms.mod['z001'] = z001.sample(1)
  parms.mod['z002'] = z002.sample(1)
  parms.mod['z003'] = z003.sample(1)
  parms.mod['e004'] = e004.sample(1)
  parms.mod['e005'] = e005.sample(1)
  parms.mod['f007004'] = parms.mod['f004007'] #f007004.sample(1)
  parms.mod['f008006'] = f008006.sample(1)
  parms.mod['f009006'] = f009006.sample(1)
  parms.mod['storage001'] = X001.sample(1)
  parms.mod['storage002'] = X002.sample(1)
  parms.mod['storage003'] = X003.sample(1)
  parms.mod['storage004'] = X004.sample(1)
  parms.mod['storage005'] = X005.sample(1)
  parms.mod['storage006'] = X006.sample(1)
  parms.mod['storage007'] = parms.mod['storage004']/10 #X007.sample(1)
  parms.mod['storage008'] = X008.sample(1)
  parms.mod['storage009'] = X009.sample(1)

  out.mod=ode(y=state.UA, times=times, func=model, parms=parms.mod)
  M.mod = odeBack2ena(out=out.mod, parms=parms.mod)
  
  #store parms and model
  out.store[[i]] = out.mod
  final.out.store[[i]] = out.mod[nrow(out.mod),]
  parms.store[[i]] = parms.mod
  M.UA[[i]] = M.mod
  
  #counter
  print(paste("run",i))
}

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#assemble distributions
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#input parameter distributions
parms.dist = do.call(rbind, parms.store)
remove=which(parms.dist[1,] == 0)
remove=c(remove, (ncol(parms.dist)-length(x.original)):ncol(parms.dist))
parms.dist = parms.dist[,-remove]

#raw storage
out.dist = do.call(rbind, final.out.store)
out.dist = out.dist[,5:ncol(out.dist)]

#flow distributions
get.flows = function(x){
  ext = as.extended(x)
  f14 = ext[1,4]
  f24 = ext[2,4]
  f54 = ext[5,4]
  f65 = ext[6,5]
  f26 = ext[2,6]
  f56 = ext[5,6]
  f35 = ext[3,5]
  f45 = ext[4,5]
  f47 = ext[4,7]
  f58 = ext[5,8]
  f59 = ext[5,9]
  z1 = ext[12,1]
  z2 = ext[12,2]
  z3 = ext[12,3]
  e4 = ext[4,10]
  e5 = ext[5,10]
  f74 = ext[7,4]
  f86 = ext[8,6]
  f96 = ext[9,6]
  
  return(c('f14'=f14, 'f24'=f24, 'f54'=f54, 'f74'=f74, 'f35'=f35, 'f45'=f45, 'f65'=f65, 
           'f26'=f26, 'f56'=f56, 'f86'=f86, 'f96'=f96, 'f47'=f47, 'f58'=f58, 'f59'=f59, 
           'z1'=z1, 'z2'=z2, 'z3'=z3,
           'e4'=e4, 'e5'=e5))
}

flows = lapply(M.UA, get.flows)
flows = do.call(rbind, flows)


#concentrations
volumes = c(10000, # pond volume (m3)
            5000,  # groundwater volume (m3)
            5000,  # soil volume (m3)
            900,   # aquatic vegetation (m3)
            500,   # terrestrial grasses (m3)
            1000)  # terrestrial shrubs (m3)

concs= as.data.frame(matrix(0, ncol = ncol(out.dist), nrow = nrow(out.dist)))
for(i in 1:ncol(out.dist)){
  concs[,i] = out.dist[,i]/volumes[i] #returns g/m3
  concs[,i] = concs[,i]*(1/99.446) #mol/g, mass of ClO4, returns mol/m3
  concs[,i] = concs[,i]*1000000 #umol/mol, returns umol/m3
  concs[,i] = concs[,i]*(1/1000) #m3/L, returns umol/L
  concs[,i] = concs[,i]*99.446 #g/mol, returns ug/L (ppb)
}

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#calculate environ outputs
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

env4 = lapply(M.UA, function(x) enaEnviron(x, input=TRUE, output=FALSE)$'4'[1:3,10])
env4 = do.call(rbind, env4)

env5 = lapply(M.UA, function(x) enaEnviron(x, input=TRUE, output=FALSE)$'5'[1:3,10])
env5 = do.call(rbind, env5)

env6 = lapply(M.UA, function(x) enaEnviron(x, input=TRUE, output=FALSE)$'6'[1:3,10])
env6 = do.call(rbind, env6)

env7 = lapply(M.UA, function(x) enaEnviron(x, input=TRUE, output=FALSE)$'7'[1:3,10])
env7 = do.call(rbind, env7)

env8 = lapply(M.UA, function(x) enaEnviron(x, input=TRUE, output=FALSE)$'8'[1:3,10])
env8 = do.call(rbind, env8)

env9 = lapply(M.UA, function(x) enaEnviron(x, input=TRUE, output=FALSE)$'9'[1:3,10])
env9 = do.call(rbind, env9)

# organisms: (% exposure from each environ needs more consideration) 
#  aquatic inverts, fish, birds, terrestrial inverts, mammals, humans
aquatic.inverts.box=env7
fish.box=env4
frogs.box=env4
birds.box=env4*.75+env6*.25
terrestrial.inverts.box=env6
mammals.box=env4*.5+env8*.25+env9*.25
humans.box=env5

nas=rep(NA,nrow(env4))
to.boxplot=cbind(aquatic.inverts.box,nas,
                 fish.box,nas,
                 frogs.box,nas,
                 birds.box,nas,
                 terrestrial.inverts.box,nas,
                 mammals.box,nas,
                 humans.box)